@@ -5,14 +5,13 @@ import itertools
 from astroid import InferenceError
 from astroid.objects import Super
 from astroid.nodes import ClassDef, ImportFrom, Attribute
-from astroid.scoped_nodes import Class as ScopedClass, Module
+from astroid.scoped_nodes import ClassDef as ScopedClass, Module
 
 from pylint.checkers.base import DocStringChecker, NameChecker
 from pylint.checkers.design_analysis import MisdesignChecker
 from pylint.checkers.classes import ClassChecker
 from pylint.checkers.newstyle import NewStyleConflictChecker
 from pylint.checkers.variables import VariablesChecker
-from pylint.__pkginfo__ import numversion as PYLINT_VERSION
 from pylint.checkers.typecheck import TypeChecker
 from pylint.checkers.variables import ScopeConsumer
 
@@ -159,6 +158,7 @@ MODELADMIN_ATTRS = {
 
 
 MODEL_ATTRS = {
+    'id',
     'DoesNotExist',
     'MultipleObjectsReturned',
     '_base_manager',
@@ -244,6 +244,8 @@ FOREIGNKEY_FIELD_ATTRS = {
 
 
 MANYTOMANY_FIELD_ATTRS = {
+    'add',
+    'clear',
     'related_name',
     'related_query_name',
     'limit_choices_to',
@@ -281,6 +283,11 @@ VIEW_ATTRS = {
 }
 
 
+FORM_ATTRS = {
+    'declared_fields',
+}
+
+
 def ignore_import_warnings_for_related_fields(orig_method, self, node):
     """
     Replaces the leave_module method on the VariablesChecker class to
@@ -413,33 +420,27 @@ def is_model_meta_subclass(node):
                'rest_framework.generics.GenericAPIView',
                'rest_framework.viewsets.ReadOnlyModelViewSet',
                'rest_framework.viewsets.ModelViewSet',
-               'django_filters.filterset.FilterSet',)
-    return node_is_subclass(node.parent, *parents)
-
-
-def is_model_factory_meta_subclass(node):
-    """Checks that node is derivative of DjangoModelFactory class."""
-    if node.name != 'Meta' or not isinstance(node.parent, ClassDef):
-        return False
-
-    parents = ('factory.django.DjangoModelFactory',
-               '.DjangoModelFactory',)
+               'django_filters.filterset.FilterSet',
+               'factory.django.DjangoModelFactory',)
     return node_is_subclass(node.parent, *parents)
 
 
 def is_model_factory(node):
-    """Checks that node is derivative of SubFactory class."""
+    """Checks that node is derivative of DjangoModelFactory or SubFactory class."""
     try:
         parent_classes = node.expr.inferred()
     except:  # noqa: E722, pylint: disable=bare-except
         return False
 
-    parents = ('factory.declarations.SubFactory',)
+    parents = ('factory.declarations.SubFactory', 'factory.django.DjangoModelFactory')
 
     for parent_class in parent_classes:
         try:
             if parent_class.qname() in parents:
                 return True
+
+            if node_is_subclass(parent_class, *parents):
+                return True
         except AttributeError:
             continue
 
@@ -583,6 +584,12 @@ def is_onetoonefield_attribute(node):
     return _attribute_is_magic(node, ONETOONE_FIELD_ATTRS, parents)
 
 
+def is_form_attribute(node):
+    """Checks that node is attribute of Form."""
+    parents = ('django.forms.forms.Form', 'django.forms.models.ModelForm')
+    return _attribute_is_magic(node, FORM_ATTRS, parents)
+
+
 def is_model_test_case_subclass(node):
     """Checks that node is derivative of TestCase class."""
     if not node.name.endswith('Test') and not isinstance(node.parent, ClassDef):
@@ -601,15 +608,17 @@ def generic_is_view_attribute(parents, attrs):
 def is_model_view_subclass_method_shouldnt_be_function(node):
     """Checks that node is get or post method of the View class."""
     if node.name not in ('get', 'post'):
-
         return False
 
     parent = node.parent
     while parent and not isinstance(parent, ScopedClass):
         parent = parent.parent
 
-    subclass = '.View'
-    return parent is not None and parent.name.endswith('View') and node_is_subclass(parent, subclass)
+    subclass = ('django.views.View',
+                'django.views.generic.View',
+                'django.views.generic.base.View',)
+
+    return parent is not None and node_is_subclass(parent, *subclass)
 
 
 def is_model_view_subclass_unused_argument(node):
@@ -628,7 +637,7 @@ def is_argument_named_request(node):
     """
     If an unused-argument is named 'request' ignore that!
     """
-    return 'request'in node.argnames()
+    return 'request' in node.argnames()
 
 
 def is_model_field_display_method(node):
@@ -716,88 +725,62 @@ def wrap(orig_method, with_method):
 def is_wsgi_application(node):
     frame = node.frame()
     return node.name == 'application' and isinstance(frame, Module) and \
-        (frame.name == 'wsgi' or frame.path.endswith('wsgi.py') or frame.file.endswith('wsgi.py'))
-
-
-# The names of some visit functions changed in this commit:
-# https://bitbucket.org/logilab/pylint/commits/c94ee95abaa5737f13b91626fe321150c0ddd140
-
-def _visit_class(checker):
-    return getattr(checker, 'visit_classdef' if PYLINT_VERSION >= (1, 5) else 'visit_class')
-
-
-def _visit_attribute(checker):
-    return getattr(checker, 'visit_attribute' if PYLINT_VERSION >= (1, 5) else 'visit_getattr')
-
-
-def _leave_class(checker):
-    return getattr(checker, 'leave_classdef' if PYLINT_VERSION >= (1, 5) else 'leave_class')
-
-
-def _leave_function(checker):
-    return getattr(checker, 'leave_functiondef' if PYLINT_VERSION >= (1, 5) else 'leave_function')
-
-
-def _visit_assignname(checker):
-    return getattr(checker, 'visit_assignname' if PYLINT_VERSION >= (1, 5) else 'visit_assname')
-
-
-def _visit_assign(checker):
-    return getattr(checker, 'visit_assign')
+        (frame.name == 'wsgi' or frame.path[0].endswith('wsgi.py') or frame.file.endswith('wsgi.py'))
 
 
 def apply_augmentations(linter):
     """Apply augmentation and suppression rules."""
-    augment_visit(linter, _visit_attribute(TypeChecker), foreign_key_sets)
-    augment_visit(linter, _visit_attribute(TypeChecker), foreign_key_ids)
-    suppress_message(linter, _visit_attribute(TypeChecker), 'no-member', is_model_field_display_method)
-    suppress_message(linter, _visit_attribute(TypeChecker), 'no-member', is_style_attribute)
+    augment_visit(linter, TypeChecker.visit_attribute, foreign_key_sets)
+    augment_visit(linter, TypeChecker.visit_attribute, foreign_key_ids)
+    suppress_message(linter, TypeChecker.visit_attribute, 'no-member', is_model_field_display_method)
+    suppress_message(linter, TypeChecker.visit_attribute, 'no-member', is_style_attribute)
 
     # supress errors when accessing magical class attributes
-    suppress_message(linter, _visit_attribute(TypeChecker), 'no-member', is_manager_attribute)
-    suppress_message(linter, _visit_attribute(TypeChecker), 'no-member', is_admin_attribute)
-    suppress_message(linter, _visit_attribute(TypeChecker), 'no-member', is_model_attribute)
-    suppress_message(linter, _visit_attribute(TypeChecker), 'no-member', is_field_attribute)
-    suppress_message(linter, _visit_attribute(TypeChecker), 'no-member', is_charfield_attribute)
-    suppress_message(linter, _visit_attribute(TypeChecker), 'no-member', is_datefield_attribute)
-    suppress_message(linter, _visit_attribute(TypeChecker), 'no-member', is_decimalfield_attribute)
-    suppress_message(linter, _visit_attribute(TypeChecker), 'no-member', is_filefield_attribute)
-    suppress_message(linter, _visit_attribute(TypeChecker), 'no-member', is_imagefield_attribute)
-    suppress_message(linter, _visit_attribute(TypeChecker), 'no-member', is_ipfield_attribute)
-    suppress_message(linter, _visit_attribute(TypeChecker), 'no-member', is_slugfield_attribute)
-    suppress_message(linter, _visit_attribute(TypeChecker), 'no-member', is_foreignkeyfield_attribute)
-    suppress_message(linter, _visit_attribute(TypeChecker), 'no-member', is_manytomanyfield_attribute)
-    suppress_message(linter, _visit_attribute(TypeChecker), 'no-member', is_onetoonefield_attribute)
+    suppress_message(linter, TypeChecker.visit_attribute, 'no-member', is_manager_attribute)
+    suppress_message(linter, TypeChecker.visit_attribute, 'no-member', is_admin_attribute)
+    suppress_message(linter, TypeChecker.visit_attribute, 'no-member', is_model_attribute)
+    suppress_message(linter, TypeChecker.visit_attribute, 'no-member', is_field_attribute)
+    suppress_message(linter, TypeChecker.visit_attribute, 'no-member', is_charfield_attribute)
+    suppress_message(linter, TypeChecker.visit_attribute, 'no-member', is_datefield_attribute)
+    suppress_message(linter, TypeChecker.visit_attribute, 'no-member', is_decimalfield_attribute)
+    suppress_message(linter, TypeChecker.visit_attribute, 'no-member', is_filefield_attribute)
+    suppress_message(linter, TypeChecker.visit_attribute, 'no-member', is_imagefield_attribute)
+    suppress_message(linter, TypeChecker.visit_attribute, 'no-member', is_ipfield_attribute)
+    suppress_message(linter, TypeChecker.visit_attribute, 'no-member', is_slugfield_attribute)
+    suppress_message(linter, TypeChecker.visit_attribute, 'no-member', is_foreignkeyfield_attribute)
+    suppress_message(linter, TypeChecker.visit_attribute, 'no-member', is_manytomanyfield_attribute)
+    suppress_message(linter, TypeChecker.visit_attribute, 'no-member', is_onetoonefield_attribute)
+    suppress_message(linter, TypeChecker.visit_attribute, 'no-member', is_form_attribute)
 
     for parents, attrs in VIEW_ATTRS:
-        suppress_message(linter, _visit_attribute(TypeChecker), 'no-member', generic_is_view_attribute(parents, attrs))
+        suppress_message(linter, TypeChecker.visit_attribute, 'no-member', generic_is_view_attribute(parents, attrs))
 
     # formviews have too many ancestors, there's nothing the user of the library can do about that
-    suppress_message(linter, _visit_class(MisdesignChecker), 'too-many-ancestors',
+    suppress_message(linter, MisdesignChecker.visit_classdef, 'too-many-ancestors',
                      is_class('django.views.generic.edit.FormView'))
 
     # model forms have no __init__ method anywhere in their bases
-    suppress_message(linter, _visit_class(ClassChecker), 'W0232', is_class('django.forms.models.ModelForm'))
+    suppress_message(linter, ClassChecker.visit_classdef, 'W0232', is_class('django.forms.models.ModelForm'))
 
     # forms implement __getitem__ but not __len__, thus raising a "Badly implemented container" warning which
     # we will suppress. NOTE: removed from pylint, https://github.com/PyCQA/pylint/issues/112
     # keeping here in case it gets re-implemented
-    # suppress_message(linter, _leave_class(MisdesignChecker), 'R0924', is_class('django.forms.forms.Form'))
-    # suppress_message(linter, _leave_class(MisdesignChecker), 'R0924', is_class('django.forms.models.ModelForm'))
+    # suppress_message(linter, _leave_classdef(MisdesignChecker), 'R0924', is_class('django.forms.forms.Form'))
+    # suppress_message(linter, _leave_classdef(MisdesignChecker), 'R0924', is_class('django.forms.models.ModelForm'))
 
     # Meta
-    suppress_message(linter, _visit_class(DocStringChecker), 'missing-docstring', is_model_meta_subclass)
-    suppress_message(linter, _visit_class(NewStyleConflictChecker), 'old-style-class', is_model_meta_subclass)
-    suppress_message(linter, _visit_class(ClassChecker), 'no-init', is_model_meta_subclass)
-    suppress_message(linter, _leave_class(MisdesignChecker), 'too-few-public-methods', is_model_meta_subclass)
-    suppress_message(linter, _visit_attribute(ClassChecker), 'protected-access', allow_meta_protected_access)
+    suppress_message(linter, DocStringChecker.visit_classdef, 'missing-docstring', is_model_meta_subclass)
+    suppress_message(linter, NewStyleConflictChecker.visit_classdef, 'old-style-class', is_model_meta_subclass)
+    suppress_message(linter, ClassChecker.visit_classdef, 'no-init', is_model_meta_subclass)
+    suppress_message(linter, MisdesignChecker.leave_classdef, 'too-few-public-methods', is_model_meta_subclass)
+    suppress_message(linter, ClassChecker.visit_attribute, 'protected-access', allow_meta_protected_access)
 
     # Media
-    suppress_message(linter, _visit_assignname(NameChecker), 'C0103', is_model_media_valid_attributes)
-    suppress_message(linter, _visit_class(DocStringChecker), 'missing-docstring', is_model_media_subclass)
-    suppress_message(linter, _visit_class(NewStyleConflictChecker), 'old-style-class', is_model_media_subclass)
-    suppress_message(linter, _visit_class(ClassChecker), 'no-init', is_model_media_subclass)
-    suppress_message(linter, _leave_class(MisdesignChecker), 'too-few-public-methods', is_model_media_subclass)
+    suppress_message(linter, NameChecker.visit_assignname, 'C0103', is_model_media_valid_attributes)
+    suppress_message(linter, DocStringChecker.visit_classdef, 'missing-docstring', is_model_media_subclass)
+    suppress_message(linter, NewStyleConflictChecker.visit_classdef, 'old-style-class', is_model_media_subclass)
+    suppress_message(linter, ClassChecker.visit_classdef, 'no-init', is_model_media_subclass)
+    suppress_message(linter, MisdesignChecker.leave_classdef, 'too-few-public-methods', is_model_media_subclass)
 
     # Admin
     # Too many public methods (40+/20)
@@ -807,31 +790,28 @@ def apply_augmentations(linter):
     # for method in node.methods():
     #     if not method.name.startswith('_'):
     #         nb_public_methods += 1
-    suppress_message(linter, _leave_class(MisdesignChecker), 'R0904', is_model_admin_subclass)
+    suppress_message(linter, MisdesignChecker.leave_classdef, 'R0904', is_model_admin_subclass)
 
     # Tests
-    suppress_message(linter, _leave_class(MisdesignChecker), 'R0904', is_model_test_case_subclass)
+    suppress_message(linter, MisdesignChecker.leave_classdef, 'R0904', is_model_test_case_subclass)
 
     # View
     # Method could be a function (get, post)
-    suppress_message(linter, _leave_function(ClassChecker), 'R0201', is_model_view_subclass_method_shouldnt_be_function)
+    suppress_message(linter, ClassChecker.leave_functiondef, 'no-self-use',
+                     is_model_view_subclass_method_shouldnt_be_function)
     # Unused argument 'request' (get, post)
-    suppress_message(linter, _leave_function(VariablesChecker), 'unused-argument',
+    suppress_message(linter, VariablesChecker.leave_functiondef, 'unused-argument',
                      is_model_view_subclass_unused_argument)
-    suppress_message(linter, _leave_function(VariablesChecker), 'unused-argument', is_argument_named_request)
+    suppress_message(linter, VariablesChecker.leave_functiondef, 'unused-argument', is_argument_named_request)
 
     # django-mptt
-    suppress_message(linter, _visit_class(DocStringChecker), 'missing-docstring', is_model_mpttmeta_subclass)
-    suppress_message(linter, _visit_class(NewStyleConflictChecker), 'old-style-class', is_model_mpttmeta_subclass)
-    suppress_message(linter, _visit_class(ClassChecker), 'W0232', is_model_mpttmeta_subclass)
-    suppress_message(linter, _leave_class(MisdesignChecker), 'too-few-public-methods', is_model_mpttmeta_subclass)
+    suppress_message(linter, DocStringChecker.visit_classdef, 'missing-docstring', is_model_mpttmeta_subclass)
+    suppress_message(linter, NewStyleConflictChecker.visit_classdef, 'old-style-class', is_model_mpttmeta_subclass)
+    suppress_message(linter, ClassChecker.visit_classdef, 'W0232', is_model_mpttmeta_subclass)
+    suppress_message(linter, MisdesignChecker.leave_classdef, 'too-few-public-methods', is_model_mpttmeta_subclass)
 
     # factory_boy's DjangoModelFactory
-    suppress_message(linter, _visit_class(DocStringChecker), 'missing-docstring', is_model_factory_meta_subclass)
-    suppress_message(linter, _visit_class(NewStyleConflictChecker), 'old-style-class', is_model_factory_meta_subclass)
-    suppress_message(linter, _visit_class(ClassChecker), 'W0232', is_model_factory_meta_subclass)
-    suppress_message(linter, _leave_class(MisdesignChecker), 'too-few-public-methods', is_model_factory_meta_subclass)
-    suppress_message(linter, _visit_attribute(TypeChecker), 'no-member', is_model_factory)
+    suppress_message(linter, TypeChecker.visit_attribute, 'no-member', is_model_factory)
 
     # ForeignKey and OneToOneField
     # Must update this in a thread safe way to support the parallel option on pylint (-j)
@@ -844,4 +824,4 @@ def apply_augmentations(linter):
     # else VariablesChecker.leave_module is already wrapped
 
     # wsgi.py
-    suppress_message(linter, _visit_assignname(NameChecker), 'invalid-name', is_wsgi_application)
+    suppress_message(linter, NameChecker.visit_assignname, 'invalid-name', is_wsgi_application)
